#include "Circle.h"

#include <sstream>

#include "Group.h"

using namespace std;

Circle::Circle(const pugi::xml_node& node, Group* parent)
    : Shape{node, parent},
      r{node.attribute("r").as_double()},
      color{node.attribute("color").as_string()} {}

Circle::Circle(std::string const& label, const double x, const double y, const double r,
               std::string const& color, Group* parent)
    : Shape{label, x, y, parent}, r{r}, color{color} {}

void Circle::clone_with_new_parent(Group* parent) {
  auto c = std::make_unique<Circle>(get_label_for_clone(), get_x(), get_y(), r, color, parent);
  if (parent != nullptr) {
    parent->add_child(std::move(c));
  }
}

std::string Circle::dump(std::string const& indent) const {
  ostringstream oss;
  oss << indent << "Circle " << Shape::dump(indent) << "r: " << r << ", "
      << "color: " << "\"" << color << "\"" << endl;
  return oss.str();
}

void Circle::draw(SFML_output* out, const double x_origin, const double y_origin) const {
  out->draw_circle(x_origin + get_x(), y_origin + get_y(), r, color);
}

void Circle::set_color(std::string const& aColor) { color = aColor; }
