#pragma once
#include <memory>
#include "symbols_and_intelligent_pointers.h"
#if SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 0
#include <map>
#endif // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 0

#include "Group.h"
#include "SymbolDefinition.h"

class Drawing {
public:
  explicit Drawing(const pugi::xml_node &node);
  void draw(SFML_output* out) const;
  [[nodiscard]] std::string dump(std::string const& indent) const;
  [[nodiscard]] Shape* find_shape_by_label(std::string_view a_label) const;
#if SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  [[nodiscard]] SymbolDefinition* find_symbol_by_label(std::string_view a_label) const;
#else // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  [[nodiscard]] std::shared_ptr<SymbolDefinition> find_symbol_by_label(const std::string &a_label);
#endif // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1

private:
  std::unique_ptr<Group> root;
#if SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  std::vector<std::unique_ptr<SymbolDefinition>> symbols;
#else // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  std::map<std::string,std::weak_ptr<SymbolDefinition>,std::less<>> symbols;
#endif // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
};
