#include <ranges>
#include <sstream>
#include "Drawing.h"

static_assert(SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 0, "SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR must have value 0");

using namespace std;

Drawing::Drawing(const pugi::xml_node& node)
{
  vector<shared_ptr<SymbolDefinition>> keep_weak_ptr_valid;
  // Initialize symbols
  for (const auto &child : node.children("SymbolDefinition")) {
    keep_weak_ptr_valid.emplace_back(make_shared<SymbolDefinition>(child));
    symbols[child.attribute("label").as_string()] = keep_weak_ptr_valid.back();
  }
  // Initialize root
  root = make_unique<Group>(node, nullptr, this);
}

void Drawing::draw(SFML_output* out) const {
  root->draw(out, 0, 0);
}

std::string Drawing::dump(std::string const& indent) const {
  ostringstream oss;
  oss << indent << "Drawing with symbols" << endl;
  for (const auto& wp : symbols | views::values) {
    oss << wp.lock()->dump(indent + "| ");
  }
  oss << indent << "Drawing with root = " << root->dump(indent);
  return oss.str();
}

Shape* Drawing::find_shape_by_label(const std::string_view a_label) const {
  // We search in symbols
  for (const auto& wp : symbols | views::values) {
    if (const auto sp = wp.lock()) {
      if (auto shape = sp->find_shape_by_label(a_label)) {
        return shape;
      }
    }
  }
  // We search in root.
  return root->find_shape_by_label(a_label);
}

shared_ptr<SymbolDefinition> Drawing::find_symbol_by_label(const std::string &a_label) {
  const auto it = symbols.find(a_label);
  if (it == symbols.end()) {
    return nullptr;
  }
  if (const auto sp = it->second.lock()) {
    return sp;
  }
  symbols.erase(a_label);
  return nullptr;
}
