#include <ranges>
#include <sstream>
#include "Drawing.h"

static_assert(SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1, "SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR must have value 1");

using namespace std;

Drawing::Drawing(const pugi::xml_node& node)
{
  // Initialize symbols
  for (const auto &child : node.children("SymbolDefinition")) {
    symbols.emplace_back(make_unique<SymbolDefinition>(child));
  }
  // Initialize root
  root = make_unique<Group>(node, nullptr, this);
}

void Drawing::draw(SFML_output* out) const {
  root->draw(out, 0, 0);
}

std::string Drawing::dump(std::string const& indent) const {
  ostringstream oss;
  oss << indent << "Drawing with symbols" << endl;
  for (const auto &symbol : symbols) {
    oss << symbol->dump(indent + "| ");
  }
  oss << indent << "Drawing with root = " << root->dump(indent);
  return oss.str();
}

Shape* Drawing::find_shape_by_label(const std::string_view a_label) const {
  // We search in symbols
  for (const auto &symbol : symbols) {
    if (const auto shape = symbol->find_shape_by_label(a_label)) {
      return shape;
    }
  }
  // We search in root.
  return root->find_shape_by_label(a_label);
}

SymbolDefinition* Drawing::find_symbol_by_label(const std::string_view a_label) const {
  for (const auto &symbol : symbols) {
    if (symbol->get_label() == a_label) {
      return symbol.get();
    }
  }
  return nullptr;
}
