#include <sstream>
#include "Shape.h"

using namespace std;

Shape::Shape(const pugi::xml_node& node, Group* parent) :
  label{node.attribute("label").as_string()},
  x{node.attribute("x").as_double()},
  y{node.attribute("y").as_double()},
  parent{parent} {
}

Shape::Shape(const std::string_view label, const double x, const double y, Group* parent)
    : label{label}, x{x}, y{y}, parent{parent} {
}

void Shape::clone() {
  clone_with_new_parent(parent);
}

std::string Shape::dump(std::string const& indent) const {
  ostringstream oss;
  oss << "\"" << label << "\", " << "x: " << x << ", " << "y: " << y << ", ";
  return oss.str();
}

std::string Shape::get_label_for_clone() const { return label + "+"; }

double Shape::get_x() const { return x; }

double Shape::get_y() const { return y; }

Shape* Shape::find_shape_by_label(string_view a_label) {
  return label == a_label ? this : nullptr;
}

void Shape::translate(int delta_x, int delta_y) {
  x += delta_x;
  y += delta_y;
}