#pragma once
#include <string>

#include "SFML_output.h"
#include "pugixml.hpp"

class Group;

class Shape {
 public:
  explicit Shape(const pugi::xml_node& node, Group* parent);
  Shape(std::string_view label, double x, double y, Group* parent);
  virtual ~Shape() = default;
  void clone();
  virtual void clone_with_new_parent(Group* parent) = 0;
  virtual void draw(SFML_output* out, double x_origin,
                    double y_origin) const = 0;
  [[nodiscard]] virtual std::string dump(std::string const& indent)
      const;  // We cannot have a default argument anymore (otherwise
              // "Clang-Tidy: Default arguments on virtual or override methods
              // are prohibited")
  [[nodiscard]] virtual Shape* find_shape_by_label(std::string_view a_label);
  [[nodiscard]] std::string get_label_for_clone() const;
  [[nodiscard]] double get_x() const;
  [[nodiscard]] double get_y() const;
  virtual void set_color(std::string const& aColor) = 0;
  void translate(int delta_x, int delta_y);

 private:
  std::string label;
  double x;
  double y;
  Group* parent;
};
