#include <sstream>
#include "Drawing.h"
#include "SymbolReference.h"

using namespace std;

SymbolReference::SymbolReference(const pugi::xml_node& node, Group* parent, Drawing *drawing) :
  Shape{node, parent},
  symbol_definition{drawing->find_symbol_by_label(node.attribute("symbol_label").as_string())}
{
  assert( symbol_definition != nullptr);
}

#if SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  SymbolReference::SymbolReference(std::string const& label, const double x, const double y, Group* parent, const SymbolDefinition *symbol_definition)
#else // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
  SymbolReference::SymbolReference(std::string const& label, double x, double y, Group *parent, shared_ptr<SymbolDefinition> symbol_definition)
#endif // SYMBOL_USES_UNIQUE_PTR_AND_NOT_SHARED_PTR == 1
    : Shape{label, x, y, parent}, symbol_definition{symbol_definition} {}

void SymbolReference::clone_with_new_parent(Group* parent) {
  auto sr = std::make_unique<SymbolReference>(get_label_for_clone(), get_x(), get_y(), parent,
                                  symbol_definition);
  if (parent != nullptr) {
    parent->add_child(std::move(sr));
  }
}

std::string SymbolReference::dump(std::string const& indent) const {
  ostringstream oss;
  oss << indent << "SymbolReference " << Shape::dump(indent) << "symbol_label: \"" << symbol_definition->get_label() << "\"" << endl;
  return oss.str();
}

void SymbolReference::draw(SFML_output* out, const double x_origin, const double y_origin) const {
  symbol_definition->draw(out, x_origin + get_x(), y_origin + get_y());
}

void SymbolReference::set_color(std::string const& aColor) {
  symbol_definition->set_color(aColor);
}
